if Code.ensure_loaded?(NimbleCSV) do
  defmodule ClickHouse.Format.TSV do
    @moduledoc """
    An implementation of the ClickHouse `TSV` format.
    """

    @behaviour ClickHouse.Format

    alias ClickHouse.DataType

    @parser __MODULE__.Parser

    NimbleCSV.define(@parser, moduledoc: false, separator: "\t", escape: "\"")

    ################################
    # ClickHouse.Format Callbacks
    ################################

    @impl ClickHouse.Format
    @spec names() :: [binary()]
    def names, do: ["TabSeparated", "TSV"]

    @impl ClickHouse.Format
    @spec decode(raw :: iodata()) :: {ClickHouse.Result.columns(), ClickHouse.Result.data()}
    def decode(raw) do
      data = @parser.parse_string(raw, skip_headers: false)
      {nil, data}
    end

    @impl ClickHouse.Format
    @spec encode(params :: list()) :: iodata()
    def encode(params) do
      params
      |> Enum.map(&encode_row/1)
      |> @parser.dump_to_iodata()
    end

    ################################
    # Private API
    ################################

    defp encode_row(row), do: Enum.map(row, &encode_value/1)

    defp encode_value(nil), do: nil
    defp encode_value(data), do: DataType.encode(data)
  end
end
