if Code.ensure_loaded?(NimbleCSV) do
  defmodule ClickHouse.Format.TSVWithNames do
    @moduledoc """
    An implementation of the ClickHouse `TSVWithNames` format.
    """

    @behaviour ClickHouse.Format

    alias ClickHouse.Format.TSV

    @parser TSV.Parser

    ################################
    # ClickHouse.Format Callbacks
    ################################

    @impl ClickHouse.Format
    @spec names() :: [binary()]
    def names, do: ["TabSeparatedWithNames", "TSVWithNames"]

    @impl ClickHouse.Format
    @spec decode(raw :: iodata()) :: {ClickHouse.Result.columns(), ClickHouse.Result.data()}
    def decode(raw) do
      [names | data] = @parser.parse_string(raw, skip_headers: false)
      columns = Enum.map(names, fn name -> {name, nil} end)
      {columns, data}
    end

    @impl ClickHouse.Format
    @spec encode(params :: list()) :: iodata()
    def encode(params), do: TSV.encode(params)
  end
end
