defmodule ClickHouse.Format.Values do
  @moduledoc """
  An implementation of the ClickHouse `Values` format.
  """

  @behaviour ClickHouse.Format

  import ClickHouse.Utils, only: [intersperse_map: 3]

  alias ClickHouse.DataType

  ################################
  # ClickHouse.Format Callbacks
  ################################

  @impl ClickHouse.Format
  @spec names() :: [binary()]
  def names, do: ["Values"]

  @impl ClickHouse.Format
  @spec decode(raw :: iodata()) :: no_return()
  def decode(_raw) do
    raise RuntimeError, "Values decode not implemented."
  end

  @impl ClickHouse.Format
  @spec encode(params :: list()) :: iodata()
  def encode(params) do
    intersperse_map(params, ",", fn row ->
      ["(", intersperse_map(row, ",", &DataType.encode/1), ")"]
    end)
  end
end
