defmodule ClickHouse.MixProject do
  use Mix.Project

  @version "0.17.2"

  def project do
    [
      app: :clickhouse,
      version: @version,
      elixir: "~> 1.11",
      start_permanent: Mix.env() == :prod,
      deps: deps(),
      description: description(),
      package: package(),
      name: "ClickHouse",
      docs: docs()
    ]
  end

  # Run "mix help compile.app" to learn about applications.
  def application do
    [
      extra_applications: [:logger]
    ]
  end

  defp description do
    """
    A ClickHouse database client.
    """
  end

  defp package do
    [
      files: ["lib", "mix.exs", "README*"],
      maintainers: ["Nicholas Sweeting"],
      licenses: ["MIT"],
      links: %{"GitHub" => "https://github.com/nsweeting/clickhouse"}
    ]
  end

  defp docs do
    [
      extras: ["README.md"],
      main: "readme",
      source_url: "https://github.com/nsweeting/clickhouse",
      groups_for_modules: [
        # ClickHouse,
        # ClickHouse.Client,
        # ClickHouse.Query,
        # ClickHouse.Query.Sigils,
        # ClickHouse.Result,
        # ClickHouse.ResultStream,
        # ClickHouse.ResultStreamChunk,
        # ClickHouse.Telemetry,
        Formats: [
          ClickHouse.Format,
          ClickHouse.Format.TSV,
          ClickHouse.Format.TSVWithNames,
          ClickHouse.Format.TSVWithNamesAndTypes,
          ClickHouse.Format.Values
        ],
        Interfaces: [
          ClickHouse.Interface,
          ClickHouse.Interface.HTTP
        ],
        DataTypes: [
          ClickHouse.DataType,
          ClickHouse.DataType.Encodable
        ],
        Errors: [
          ClickHouse.ConnectionError,
          ClickHouse.CoordinationError,
          ClickHouse.DatabaseError,
          ClickHouse.NetworkError,
          ClickHouse.ParsingError,
          ClickHouse.ResultStreamError
        ]
      ]
    ]
  end

  # Run "mix help deps" to learn about dependencies.
  defp deps do
    [
      {:hackney, "~> 1.17"},
      {:keyword_validator, "~> 1.0"},
      {:telemetry, "~> 0.4", optional: true},
      {:nimble_csv, "~> 1.1", optional: true},
      {:ex_doc, ">= 0.0.0", only: :dev, runtime: false}
    ]
  end
end
